#!/bin/sh
#================================================================================
# (C)2013-2018 dMajo
# Title        : ad-blocker.sh
# Version      : 2.03.0118 - 009ß1
# Author       : dMajo (http://forum.synology.com/enu/memberlist.php?mode=viewprofile&u=69661)
# Description  : Script to block add-banner servers, dns based
# Dependencies : Syno DNSServer package, bash V4 or higher, sed, sort, wget
# Usage        : sh ad-blocker.sh
#--------------------------------------------------------------------------------
# Version history:
# 2013.09.01 - 1.00.0001:	Initial release
# 2013.09.08 - 1.00.0004:	Fix: changed include target to support views
# 2013.09.12 - 1.00.0005:	Added automatic zone file generation and some basic error handling
# 2014.03.29 - 1.01.0013:	Added dependencies check
# 2014.03.30 - 1.02.0017:	Script reorganized
# 2014.04.06 - 1.02.0018:	Fix: fixed serial number in zone file generation
# 2015.11.01 - 1.03.0031:	Added additional blacklist sources (users request)
# 2015.11.02 - 1.04.0035:	Added blacklist&whitelist filo option
# 2015.12.26 - 1.06.0041:	Added check for existing dns zones (thanks Briolet)
# 2016.04.24 - 2.00.0051:	Script refactoring with external config file
# 2016.04.25 - 2.00.0053:	Change: config file structure
# 2016.04.30 - 2.01.0061:	Fix: config file security vulnerability and minor bugs
# 2016.05.01 - 2.01.0072:	Script reorganization and cleanup
# 2017.12.16 - 2.02.0086:	Compatibility for new DSNServer version
# 2017.12.24 - 2.03.0101:	Improved zone SN handling and fixed logging feature after Synology switch to DB
# 2018.01.07 - 2.03.0102:	Fixed small bug in Type=2 data polishing routine (thanks M4RC)
# 2018.01.27 - 2.03.0103:	Improved ad-blocker.db's header comments
# 2018.01.27 - 2.03.0105:	Fix: improved capitals handling and fixed a minor copy&paste bug (thanks Briolet)
# 2018.01.28 - 2.03.0111:	Improved logging and added IPv6 support
# 2018.01.29 - 2.03.0112:	Fixed a small DB logging glitch (thanks Briolet)
# 2018.02.04 - 2.03.0115:	Enhanced cleaning filter rules
# 2018.02.04 - 2.03.0118:	Added User BL & WL comments handling
#
# Changes by Briolet:
# 2018.02.04 - 2.03.0118 - 001:	Changed some paths for private use
#
# Changes by King006:
# 2018.03.18 - 2.03.0118 - 002:	Make changes for ad-blocker.conf
#				Check if this file is already running (if yes, then stop and log)
#				Options for Secondary DNS
#				Make layout changes and log process ID for debugging (if this script is running two times)
# 2018.03.19 - 2.03.0118 - 003:	Remove option check if the file contains something we don't want
#				Change debug mode (now you can enable debug mode only for tmpdir, log or final-list.txt)
# 2018.03.20 - 2.03.0118 - 004:	Make changes for task e-mails (mail is empty by default, it's a bug)
#		                Extra logging added for the e-mail function through the task scheduler
# 2018.03.21 - 2.03.0118 - 005:	Checks added if the values ​​in ad-blocker.conf have been entered correctly
#				Add custom value IPv4_status and IPv6_status to ad-blocker.conf (manually enable or disable IPv4 and IPv6)
#
# Changes by Briolet:
# 2018.03.21 - 2.03.0118 - 006: If a list can not be downloaded, the old list is now retained. (otherwise he is empty)
# 2018.03.27 - 2.03.0118 - 007: Added check for correct serial format
# 2018.04.01 - 2.03.0118 - 008: Added timestamping. Blocklists are not downloaded if there is no newer version. This
#				only works for sites that add timestamps to the blocklists.
# 2019.01.01 - 2.03.0118 - 009: write out a file with changes between lists.
# 2019.09.01 - 2.03.0118 - 010: the path to the scripfolder now can contain spaces.
# 2021.10.17 - 2.03.0118 - 011: Added code to make it work with DSM 7.
#================================================================================
# Ad-Blocker Config file (see ad-blocker.conf)
#--------------------------------------------------------------------------------
# User whitelist and blacklist path/file association.
# ---------------------------------------------------
# User's black/white lists are supported as a FQDN domains/hosts list, one per line, same as downloads of BL_Type=3
# USER_BL = /path/to/the/filename
# USER_WL = /path/to/the/filename
#--------------------------------------------------------------------------------
# Blacklist array definition
# --------------------------
# Blacklist configuration fields, indexing starts from 1
# BL_Items	= Number of total configured blacklist downloads (n=1..Items)
#
# BL_Name[n]	= Blacklist's friendly name
# BL_URL[n]	= URL address and parameters to retrive the ASCII BL test file
# BL_Type[n]	= 0 Synology zone file (no processing, taken as is)
#		  1 MSDNS/Bind standard zone file
#		  2 Standard hosts file
#		  3 FQDN domain/host(only) list
# BL_Zipped[n]	= 0/1 Indicates that download is single zipped file
# BL_Enabled[n]	= 0/1 If not enabled will not be processed
#================================================================================

	# Check if we can use bash. Only available since DSM 6.0
	if [ "${BASH_VERSION}" == "" ] ; then
		echo "No bash available on this nas, aborting."
		echo "Update to DSM 6.x or newer to run this script."
		exit 1;
	fi

	# Including config file, you can make changes there.
	# source $(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/ad-blocker.conf
	source "$(dirname "$0")"/ad-blocker.conf

    # DSM 7 wants the full paths
    source /etc/VERSION
    if [ $majorversion -eq 7 ]; then
        DSM7="${RootDir}/named"
        DSM7sed="\/var\/packages\/DNSServer\/target\/named"
    else
        DSM7=""
        DSM7sed=""
    fi
    
    if [ $majorversion -gt 7 ]; then
        echo "Stopping script because of yet unsupported DSM version "$majorversion
        Exit 1;
    fi

	# Log settings
	function LogThis(){ # Parameters: LogMessage[text], Abort[0/1], LogDestination[toDNS/toMail/toAll], LogSeverity[-1 .. -5]
		if [[ $LogToDNS -eq 1 ]] && [[ ($3 = "toDNS" ||  $3 = "toAll") ]] && [[ $1 != "" ]]; then
			if [ "$(printf "2.2.1-3051\n$DnsVersion" | sort -V | head -n1)" == "$DnsVersion" ] && [ "$DnsVersion" != "2.2.1-3051" ]; then
				#Log to file up to 2.2.1-3051
				DTStamp=$(date +"%d-%b-%Y %T.%3N")
				echo "${DTStamp} AdBlock: $1" >> ${LogFile}
			else
				#Log to DB above to 2.2.1-3051
				DTStamp=$(date +"%Y/%m/%d %T.%3N") && LogMsg=$1 && LogMsg=${LogMsg//\'/\'\'}
				# $4=Severity: Info=-1, Notice=-2 ,Warning=-3, Error=-4, Critical=-5
				sqlite3 $LogDB "INSERT INTO LOGS (TIME,TAG,CATAGORY,MODULE,SEVERITY,CONTENT) VALUES ('${DTStamp}','','adblock','ad-blocker.sh','$4','AdBlocker: ${LogMsg}');"
			fi
		fi

		DTStamp=$(date +"%Y-%m-%d %T")
		if [[ $1 = "" ]]; then
			while IFS= read -r line; do
				echo "--${DTStamp}--  $line" >> ${ScriptLog}
			done
		else
			echo "--${DTStamp}--  ad-blocker.sh: $1" >> ${ScriptLog}

			if [[ $3 = "toMail" ]] || [[ $3 = "toAll" ]]; then
				echo "ad-blocker.sh: $1"
			fi

			if [ $2 -eq 1 ]; then
				ExitScript 1
			fi
		fi
	}

	function ExitScript(){ # Parameters: Abort[0/1]
		LogToDNS=1
		if [ $1 -eq 0 ]; then
			if [ "$Debugtmp" -eq 0 ]; then
				LogThis "Remove ${TmpDir}" 0
				rm -rfv "${TmpDir}" > /dev/null
			fi

			# Stopping script
			LogThis "Stopping script with ID $$" 0 "toMail"

			# Cleanup ad-blocker.log
			if [ "$Debuglog" -eq 0 ]; then
				LogThis "Remove ${ScriptLog}" 0
				rm -f "${ScriptLog}" > /dev/null
			fi

		else
			LogThis "Update aborted with errors, see update log for details ${ScriptLog}" 0 "toAll" -4
			LogThis "Stopping script with ID $$" 0 "toMail"
		fi

		exit $1
	}

	# start with an empty log
	rm -fv "${ScriptLog}" | LogThis

	# Starting script
	LogThis "Starting script with ID $$" 0 "toMail"

	# Check if this script is already running. If yes, then stop and log.
	status=`ps -efww | grep -w "ad-blocker.sh" | grep -v grep | grep -v $$ | awk '{ print $2 }'`
	if [ ! -z "$status" ]; then
		LogThis "Process is already running. Skipping this process" 0 "toMail"
	exit 1;
	fi

	# Check values in ad-blocker.conf
	if [ -z "${ScriptCfg}" ]; then
		LogThis "Please make sure there is a file ad-blocker.conf in the scriptfolder" 0 "toMail"
	exit 1;
	fi
	
	if [ -z "${Secdns}" ]; then
		LogThis "Please set secdns to 0 or 1 in "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	if [ -z "${LogToDNS}" ]; then
		LogThis "Please set LogToDNS to 0 or 1 in "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	if [ -z "${Debugtmp}" ]; then
		LogThis "Please set Debugtmp to 0 or 1 in "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	if [ -z "${Debugfinallist}" ]; then
		LogThis "Please set Debugfinallist to 0 or 1 in "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	if [[ "$Secdns" -eq 1 ]]; then
		if [ "${Secdns_path}" == "" ] || [ ! -d ${Secdns_path} ]; then
			LogThis "Please set secdns_path to a valid existing path in "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
		exit 1;
		fi
	fi

	if [ "${DnsOutputfile}" == "" ]; then
		LogThis "Please set DnsOutputfile in "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	# Check if IPv4 is set correctly
	if [[ $IPv4 =~ ^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$ ]]; then
		LogThis "The IPv4 IP Address is correct. The procedure continues" 0
	else
		LogThis "he IPv4 IP Address isn't correct. Please check "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	# Check if IPv6 is set correctly
	regex='^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$'
	if [[ $IPv6 =~ $regex ]]; then
		LogThis "The IPv6 IP Address is correct. The procedure continues" 0
	else
		LogThis "The IPv6 Address isn't correct. Please check "${ScriptDir}"/ad-blocker.conf" 0 "toMail"
	exit 1;
	fi

	# Check if needed dependencies exists
	MissingDep=0
	for NeededDep in 7z cat chown cut date grep mv rm sed sort uniq wget; do
		if ! hash "$NeededDep" >/dev/null 2>&1; then
			LogThis "Command not found in PATH: ${NeededDep}" 0
			MissingDep=$((MissingDep+1))
		fi
	done

	if [ $MissingDep -gt 0 ]; then
		LogThis "Minimum ${MissingDep} commands are missing in PATH, aborting" 1 "toMail"
	fi

	if [ -f "$PackageDir/enabled" ]; then
		DnsVersion="$(grep -oP '(?<=version\=\").*(?=\")' ${PackageDir}/INFO)"
	else
		LogThis "DNS Server package not installed, aborting" 1 "toMail"
	fi

	#Read user config
	#Config sets LogToDNS flag till the exit script function
	LogToDNS=1 ; LogThis "Update started by user: $SUDO_USER $(whoami) ..." 0 "toDNS" -2

	#Create a directory for the sourcefiles
	if [ ! -d "${SourceDir}" ]; then
		LogThis "Create a directory for the source files" 0
		mkdir -pv "${SourceDir}" | LogThis
	fi

	for (( BL_Index=1; BL_Index<=$BL_Items; BL_Index++ )); do
		if [ ! -d "${SourceDir}"/${BL_Index} ] && [ ${BL_Enabled[${BL_Index}]} -gt 0 ]; then
			mkdir -pv "${SourceDir}"/${BL_Index} | LogThis
		fi
	done

	#Download blacklists and build raw files to temp folder
	LogThis "Download blacklists and build raw files to temp folder" 0
	mkdir -pv "${TmpDir}" | LogThis
	cd "${TmpDir}"

	if [ $? -ne 0 ]; then
		LogThis "Can't change directory to ${TmpDir}" 1 "toMail"
	fi
	rm -fv adbListType*.* | LogThis
	rm -fv adbSourceURLs | LogThis

	if [ -f "${USER_BL}" ]; then
		LogThis "Building combined blacklist ..." 0 "toMail"
		sed -r -e 's/[[:blank:]]*[\#\/]+[[:print:]]+//g' "${USER_BL}" > adbListType3.raw
		echo >> adbListType3.raw ## make sure it end with a new line
	fi

	for (( BL_Index=1; BL_Index<=$BL_Items; BL_Index++ )); do if [ ${BL_Enabled[${BL_Index}]} -gt 0 ]; then
		LogThis "Downloading ${BL_Name[${BL_Index}]} ..." 0 "toAll" -2
		echo "ad-blocker.sh: Source[${BL_Index}]: ${BL_URL[${BL_Index}]}" >> adbSourceURLs
		cd "${SourceDir}"/${BL_Index}

		# Check if a file is present and not a leftover from a different config file
		DownloadedFile=$(basename ${BL_URL[${BL_Index}]})
		LogThis "Basename from url: ${DownloadedFile}" 0
		
		if [ ! -f "${DownloadedFile}" ]; then
			DownloadedFile=$(ls -S | head -1) # Check if this file is our previous download
			LogThis "File in folder: ${DownloadedFile}, File to fetch: $(basename ${BL_URL[${BL_Index}]})" 0
			if [[ "${DownloadedFile}" != "" && "${DownloadedFile}" != *"$(basename ${BL_URL[${BL_Index}]})" ]] ; then
				# For some url's, like the Yoyo list, the filename contains part of the path with escaped %2F
				# When opening a file from a Mac, the hidden @eaDir can be created. Therefor sort the listing
				rm -rfv "${SourceDir}"/${BL_Index}/*.* | LogThis
				LogThis "Clearing folder ${BL_Index} because it contains a wrong file" 0
				DownloadedFile=""
			fi
		fi

			wget -N -a "${ScriptLog}" "${BL_URL[${BL_Index}]}"

		if [[ $? -ne 0 ]] && [[ ${#DownloadedFile} -eq 0 ]]; then
		LogToDNS=1
			# Abort on error to prevent continuing with empty blacklists.
			# It's better to stay with the previous list.
			# Script exits on above logging
			LogThis "Error on downloading ${BL_Name[${BL_Index}]}" 1 "toAll" -3
		fi
		
		# Processing blocklist
		if [ $? -ne 0 ]; then
			LogThis "Error on downloading ${BL_Name[${BL_Index}]}, continuing with previous list" 0 "toAll" -2
		fi
		
		if [ -f "$DownloadedFile" ]; then
			cp ${DownloadedFile} "${TmpDir}"/adbList${BL_Index}.dnl | LogThis
		else
			cp $(ls -S | head -1) "${TmpDir}"/adbList${BL_Index}.dnl | LogThis
		fi
		
		cd "${TmpDir}"

			# Unzip if necessary
		if [ ${BL_Zipped[${BL_Index}]} -gt 0 ]; then
			LogThis "Unzipping ${BL_Name[${BL_Index}]} into folder '${BL_Index}' ..." 0
			7z e -o${BL_Index} adbList${BL_Index}.dnl | LogThis
			mv -fv ${BL_Index}/* adbList${BL_Index}.raw | LogThis
		else
			mv -fv "${TmpDir}/adbList${BL_Index}.dnl" "${TmpDir}/adbList${BL_Index}.raw" | LogThis
		fi
		LogThis "Linking by tipology '${BL_Name[${BL_Index}]}' to 'adbListType${BL_Type[${BL_Index}]}.raw' ..." 0
		echo "//===== ${BL_Name[${BL_Index}]} ====================================================================" | cut -c1-62 >> "adbListType${BL_Type[${BL_Index}]}.raw"
		cat "${TmpDir}/adbList${BL_Index}.raw" >> "adbListType${BL_Type[${BL_Index}]}.raw"
	fi done

	#Downloaded file conversion - Place to add additional file formats (everything will follow)
	LogThis "Cleaning downloaded files ..." 0 "toAll" -2
	rm -fv adbListAll.raw | LogThis
	for BL_Index in {0..3}; do if [ -f adbListType$BL_Index.raw ]; then
		#--------------------------------------------------------------------------------------
		case $BL_Index in ### Additional download file format decoders can be added here ###
		[0-1]) LogThis "Cleaning type $BL_Index files ..." 0 # [0-1]standard zone file
		grep -o -E '^[[:blank:]]*zone[[:blank:]]*"([A-Za-z0-9]|_|\-|\~|\.)+\.[A-Za-z]{2,}' adbListType$BL_Index.raw >> adbListAll.raw # Extract domains (2018.02.04)
		;;
		2) LogThis "Cleaning type 2 files ..." 0 # [2]Standard hosts file
		sed -r -n -e '/(localhost|local[[:blank:]]*\n|[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.255|0\.0\.0\.0[[:blank:]]+0\.0\.0\.0)/d' -e '/^(127\.0\.0\.1|0\.0\.0\.0)[[:blank:]]+(\w|\-|\~|\.)+\.[A-Za-z]{2,}/p' adbListType$BL_Index.raw > adbListType$BL_Index.p01 # Clean and Validate (2018.02.04)
		sed -r -e 's/[[:blank:]]*[\#\/].*$//g' adbListType$BL_Index.p01 > adbListType$BL_Index.p02 # remove comment
		sed -r -e 's/^(127\.0\.0\.1|0\.0\.0\.0)[[:space:]]*/zone "/g' adbListType$BL_Index.p02 >> adbListAll.raw # Add line head (2018.01.07 M4RC)
		;;
		3) LogThis "Cleaning type 3 files ..." 0 # [3]FQDN domain/host(only) list
		sed -r -n -e '/^[[:space:]]*(\w|\-|\~|\.)+\.[A-Za-z]{2,}/p' adbListType$BL_Index.raw > adbListType$BL_Index.p01 # Clean (2018.02.04)
		sed -r -e 's/^[[:space:]]*/zone "/g' adbListType$BL_Index.p01 >> adbListAll.raw # Add line head
		;;
		esac
		#--------------------------------------------------------------------------------------
	fi done

	# Patch uppercase chars and reduces duplicated lines to single line
	sed -e 's/\s*$//g' adbListAll.raw | sed -e 's/\([[:upper:]]\)/\L\1/g' | sort | uniq > adbListAll.all #(2018.01.27 Briolet)

	# Whitelist generation and processing
	LogThis "Building combined whitelist ..." 0 "toMail"
	rm -fv adbWhiteList.* | LogThis
	echo "----- DNS Server defined zones ----------------------" > adbWhiteList.raw
	for SynoZone in "$ZoneDataDir"/* ;do
		sed -r -e 's/(^.*\/)|\([0-9]*\)$//g' <<< $SynoZone >> adbWhiteList.p01
	done

	sed -e 's/\([[:upper:]]\)/\L\1/g' adbWhiteList.p01 | sort | uniq >> adbWhiteList.raw # Cleaning and sorting (2018.01.27 Briolet)
	if [ -f "${USER_WL}" ]; then
		echo "----- USER Whitelist entries ------------------------" >> adbWhiteList.raw
		sed -r -e 's/[[:blank:]]*[\#\/]+[[:print:]]+//g' "${USER_WL}" | sed -e 's/\([[:upper:]]\)/\L\1/g' | sort | uniq -i >> adbWhiteList.raw #Cleaning and sorting (2018.01.27 Briolet)
	fi

	sed -r -n -e '/ad-blocker.db|null.zone.file/{d;n;}' -e '/^[[:space:]]*([A-Za-z0-9]|-|_|\.)*\.([A-Za-z])*/p' adbWhiteList.raw > adbWhiteList.p01 # Clean and Validate

	#The following lines are added twice to prevent "whitelist" becoming "blacklist" in case off missing downloads/correspondences
	sed -r -e 's/^[[:space:]]*/zone "/g' adbWhiteList.p01 >> adbListAll.all # Add line head (once)
	sed -r -e 's/^[[:space:]]*/zone "/g' adbWhiteList.p01 >> adbListAll.all # Add line head (twice)

	#Database file generation

	LogThis "Building final zones database ..." 0 "toAll" -2
	echo "=====================================================" > ad-blocker.raw
	echo "===== Ad-Blocker.db =================================" >> ad-blocker.raw
	echo "=====================================================" >> ad-blocker.raw
	echo "Generated automatically by Ad-Blocker.sh" >> ad-blocker.raw
	echo "Build timestamp: ${DTStamp}" >> ad-blocker.raw
	echo >> ad-blocker.raw
	echo "Together with the 'null.zone.file' records this DB" >> ad-blocker.raw
	echo "will return "${IPv4}" for the listed domains/hosts" >> ad-blocker.raw
	echo "will return "${IPv6}" for the listed domains/hosts" >> ad-blocker.raw
	echo >> ad-blocker.raw
	cat adbWhiteList.raw >> ad-blocker.raw
	echo "----- USER Blacklist entries ------------------------" >> ad-blocker.raw
	sed -r -e 's/[[:blank:]]*[\#\/].*$//g' "${USER_BL}" | sed -r -e '/^\s*$/d' | sort >> ad-blocker.raw
	echo >> ad-blocker.raw
	echo "----- Used source URLs ------------------------------" >> ad-blocker.raw
	cat adbSourceURLs >> ad-blocker.raw
	echo "=====================================================" >> ad-blocker.raw
	sed -e 's/^/\/\//g' ad-blocker.raw > ad-blocker.db
	echo >> ad-blocker.db
	commentcount=$(wc -l < ad-blocker.db)

	# Remove duplicate pairs whitelistening the DB and add the "suitable line tails" (mainly file path for Synology bind  "/etc/zone/master/null.zone.file")
	sort adbListAll.all | uniq -u | sed -e 's/\.*[[:space:]]*$/" { type master; notify no; file "'${DSM7sed}'\/etc\/zone\/master\/null.zone.file"; allow-transfer {none;}; allow-update {none;}; allow-query {any;}; };/g' >> ad-blocker.db
	
    # Only create the database. Don't change the live database or null.zone file
	if [ "$Testing" -eq 1 ]; then
	    LogThis "Aborting because of testmode. Check ad-blocker.db file in ad-blocker.tmp folder" 0 "toMail"
	    ExitScript 0
	fi

	# Place the DB to the right place
	LogThis "Place ad-blocker.db to the right place" 0
	chown -v DNSServer:DNSServer ad-blocker.db | LogThis
	if [ $? -eq 0 ] && [ -f ad-blocker.db ]; then
		chmod -v 0644 ad-blocker.db | LogThis
		mv -fv ad-blocker.db ${ZoneDataDir}/ad-blocker.db | LogThis
		if [ "$Debugfinallist" -eq 1 ]; then
			if [ -f "${ScriptDir}"/ad-final-list.txt ]; then 
				mv -fv "${ScriptDir}"/ad-final-list.txt "${ScriptDir}"/ad-final-list-old.txt | LogThis
			else
				echo "" > "${ScriptDir}"/ad-final-list-old.txt
			fi
			cp -fv ${ZoneDataDir}/ad-blocker.db "${ScriptDir}"/ad-final-list.txt | LogThis # Copy list to script dir for user reading
			diff "${ScriptDir}"/ad-final-list-old.txt "${ScriptDir}"/ad-final-list.txt > "${ScriptDir}"/ad-final-changes.txt
			rm -fv "${ScriptDir}"/ad-final-list-old.txt | LogThis
		fi
		domaincount=$(($(wc -l < ${ZoneDataDir}/ad-blocker.db) - $commentcount))
		LogThis "Added ${domaincount} domains and subdomains to the blocklist" 0 "toMail"
	else
		LogThis "Script requires root permissions" 1
	fi

	# Include the new zone data
	LogThis "Updating DNS Server files ..." 0
	cd ${ZoneDataDir}
	if [ $? -ne 0 ]; then
		LogThis "Can't change directory to ${ZoneDataDir}" 1 "toMail"
	fi

	if [ -f ad-blocker.db ] && [ -f null.zone.file ]; then
		LogThis "Rebuild data null.zone.file" 0
		# Rebuild data null.zone.file
		echo 'zone "null.zone.file" {'					> null.zone.file
		echo '	type master;'						>> null.zone.file
		echo '	file "'${DSM7}'/etc/zone/master/null.zone.file";'		>> null.zone.file
		echo '	allow-transfer {any;};'					>> null.zone.file
		echo '	allow-update {none;};'					>> null.zone.file
		echo '	allow-query {any;};'					>> null.zone.file
		echo '};'							>> null.zone.file
		echo 'include "'${DSM7}'/etc/zone/data/ad-blocker.db";'			>> null.zone.file
		chown -v DNSServer:DNSServer null.zone.file | LogThis
		chmod -v 0644 null.zone.file | LogThis

		# Rebuild master null.zone.file
		cd ${ZoneMasterDir}
		if [ $? -ne 0 ]; then
			LogThis "Can't change directory to ${ZoneMasterDir}" 1 "toMail"
		fi

		CurrSN=$(sed -nr '/@[[:blank:]+]IN[[:blank:]+]SOA|null\.zone\.file\.[[:blank:]+]IN[[:blank:]+]SOA/{/[0-9]/!N;s/[^0-9]+([0-9]+).*/\1/p}' 'null.zone.file')
		if [[ "${CurrSN}" -eq "1" || ${#CurrSN} -ne 10 ]]; then
		    LogThis "Wrong serial format detected for the null.zone.file" 0 "toAll"
		    LogThis "Delete the null.zone.file and create a new one with a date as serial format" 1 "toAll"
		fi

		Now=$(date +%Y%m%d)
		LogThis "Rebuild master null.zone.file ..." 0
		if [[ "${Now}" -ne "${CurrSN::-2}" || ${#CurrSN} -lt ${#Now} ]]; then
			ZoneSN="${Now}00" #Set new timestamp
		else
			ZoneSN="${Now}$(printf '%02d' $((10#${CurrSN: -2} + 1)) )" # Increment if already same timestamp
		fi

		echo '$TTL 86400				; one day'			> null.zone.file
		echo '@ IN	SOA	ns.null.zone.file. mail.null.zone.file. (' 		>> null.zone.file
		echo '			'${ZoneSN}'	; serial number YYYYMMDDNN'		>> null.zone.file
		echo '			86400		; refresh	1 day'			>> null.zone.file
		echo '			7200		; retry		2 hours'		>> null.zone.file
		echo '			864000		; expire	10 days'		>> null.zone.file
		echo '			86400 )		; min ttl	1 day'			>> null.zone.file
		echo '		NS	ns.null.zone.file.'					>> null.zone.file

        if [ "$IPv4_status" == "ON" ]; then
            echo '		A	'${IPv4}''						>> null.zone.file
            LogThis "Add IPv4 record to null.zone.file" 0
        fi

        if [ "$IPv6_status" == "ON" ]; then
            echo '		AAAA	'${IPv6}''						>> null.zone.file
            LogThis "Add IPv6 record to null.zone.file" 0
        fi

        if [ "$IPv4_status" == "ON" ]; then
            echo '* IN	A	'${IPv4}''						>> null.zone.file
            LogThis "Add wildcard IPv4 record to null.zone.file" 0
        fi

        if [ "$IPv6_status" == "ON" ]; then
            echo '* IN	AAAA	'${IPv6}''						>> null.zone.file
            LogThis "Add wildcard IPv6 record to null.zone.file" 0
        fi

        chown -v DNSServer:DNSServer null.zone.file | LogThis
        chmod -v 0644 null.zone.file | LogThis
        LogThis "null.zone.file SN changed from ${CurrSN} to ${ZoneSN}" 0
    else
        LogThis "*******************************************************************" 1 "toMail" -4
        LogThis "File not found, check for presence of null.zone.file domain zone" 1 "toAll" -4
        LogThis "Open DNS Server package and create a new masterzone: null.zone.file" 1 "toMail" -4
        LogThis "*******************************************************************" 1 "toMail" -4
    fi

    # Reload the server config after modifications
    if [ $majorversion -eq 7 ]; then
        # DSM 7
        if [[ "$(/usr/syno/bin/synopkg status DNSServer)" == *"started"*  ]]; then
            LogThis "Restarting DNS Server ..." 0
            /usr/syno/bin/synopkg restart DNSServer | LogThis
        else
            LogThis "DNS Server not started ..." 0 "toMail"
        fi
    else
        # DSM 5 or 6
        if ${PackageDir}/scripts/start-stop-status status; then
            LogThis "Restarting DNS Server ..." 0
            ${RootDir}/script/reload.sh | LogThis
        else
            LogThis "DNS Server not started ..." 0 "toMail"
        fi
    fi
    
    # The loging database of DNS Server is sometimes locked when restarting with synopkg
    if [ $majorversion -eq 7 ]; then
        LogThis "Update completed successfully SN ${ZoneSN}" 0 "toMail" -2
    else
        LogThis "Update completed successfully SN ${ZoneSN}" 0 "toAll" -2
    fi

    # Copy ad-blocker.db and null.zone.file to the secundary DNS directory.
    # Please make this directory on your NAS! (for secondary DNS if enabled in ad-blocker.conf)
    if [[ "$Secdns" -eq 1 ]]; then
        LogThis "Copy ${DnsOutputfile} and null.zone.file to directory $(basename ${Secdns_path})" 0 "toMail"
        cp -Rp ${ZoneDataDir}/ad-blocker.db ${Secdns_path}/${DnsOutputfile}
        cp -Rp ${ZoneMasterDir}/null.zone.file ${Secdns_path}
    fi

	ExitScript 0
